@@ -1,22 +1,33 @@
 from __future__ import division
 
+import array
 import os
 import subprocess
 from tempfile import TemporaryFile, NamedTemporaryFile
 import wave
 import sys
+import struct
 from .logging_utils import log_conversion
+import base64
 
 try:
     from StringIO import StringIO
 except:
-    from io import StringIO, BytesIO
+    from io import StringIO
+
+from io import BytesIO
+
+try:
+    from itertools import izip
+except:
+    izip = zip
 
 from .utils import (
     _fd_or_path_or_tempfile,
     db_to_float,
     ratio_to_db,
     get_encoder_name,
+    get_array_type,
     audioop,
 )
 from .exceptions import (
@@ -25,13 +36,15 @@ from .exceptions import (
     InvalidID3TagVersion,
     InvalidTag,
     CouldntDecodeError,
+    CouldntEncodeError,
+    MissingAudioParameter,
 )
 
 if sys.version_info >= (3, 0):
     basestring = str
     xrange = range
     StringIO = BytesIO
-    
+
 
 class ClassPropertyDescriptor(object):
 
@@ -54,7 +67,7 @@ class ClassPropertyDescriptor(object):
         if not isinstance(func, (classmethod, staticmethod)):
             func = classmethod(func)
         self.fset = func
-        return self    
+        return self
 
 def classproperty(func):
     if not isinstance(func, (classmethod, staticmethod)):
@@ -87,7 +100,7 @@ class AudioSegment(object):
     @classproperty
     def ffmpeg(cls):
         return cls.converter
-    
+
     @ffmpeg.setter
     def ffmpeg(cls, val):
         cls.converter = val
@@ -97,14 +110,33 @@ class AudioSegment(object):
     }
 
     def __init__(self, data=None, *args, **kwargs):
-        if kwargs.get('metadata', False):
+        self.sample_width = kwargs.pop("sample_width", None)
+        self.frame_rate = kwargs.pop("frame_rate", None)
+        self.channels = kwargs.pop("channels", None)
+
+        audio_params = (self.sample_width, self.frame_rate, self.channels)
+
+        # prevent partial specification of arguments
+        if any(audio_params) and None in audio_params:
+            raise MissingAudioParameter("Either all audio parameters or no parameter must be specified")
+
+        # all arguments are given
+        elif self.sample_width is not None:
+            if len(data) % (self.sample_width * self.channels) != 0:
+                raise ValueError("data length must be a multiple of '(sample_width * channels)'")
+
+            self.frame_width = self.channels * self.sample_width
+            self._data = data
+
+        # keep support for 'metadata' until audio params are used everywhere
+        elif kwargs.get('metadata', False):
             # internal use only
             self._data = data
             for attr, val in kwargs.pop('metadata').items():
                 setattr(self, attr, val)
         else:
             # normal construction
-            data = data if isinstance(data, basestring) else data.read()
+            data = data if isinstance(data, (basestring, bytes)) else data.read()
             raw = wave.open(StringIO(data), 'rb')
 
             raw.rewind()
@@ -114,10 +146,55 @@ class AudioSegment(object):
             self.frame_width = self.channels * self.sample_width
 
             raw.rewind()
-            self._data = raw.readframes(float('inf'))
+
+            # the "or b''" base case is a work-around for a python 3.4
+            # see https://github.com/jiaaro/pydub/pull/107
+            self._data = raw.readframes(float('inf')) or b''
+
+        # Convert 24-bit audio to 32-bit audio.
+        # (stdlib audioop and array modules do not support 24-bit data)
+        if self.sample_width == 3:
+            byte_buffer = BytesIO()
+
+            # Workaround for python 2 vs python 3. _data in 2.x are length-1 strings,
+            # And in 3.x are ints.
+            pack_fmt = 'BBB' if isinstance(self._data[0], int) else 'ccc'
+
+            # This conversion maintains the 24 bit values.  The values are
+            # not scaled up to the 32 bit range.  Other conversions could be
+            # implemented.
+            i = iter(self._data)
+            padding = {False: b'\x00', True: b'\xFF'}
+            for b0, b1, b2 in izip(i, i, i):
+                byte_buffer.write(padding[b2 > b'\x7f'[0]])
+                old_bytes = struct.pack(pack_fmt, b0, b1, b2)
+                byte_buffer.write(old_bytes)
+
+
+            self._data = byte_buffer.getvalue()
+            self.sample_width = 4
+            self.frame_width = self.channels * self.sample_width
 
         super(AudioSegment, self).__init__(*args, **kwargs)
 
+    @property
+    def raw_data(self):
+        """
+        public access to the raw audio data as a bytestring
+        """
+        return self._data
+
+
+    def get_array_of_samples(self):
+        """
+        returns the raw_data as an array of samples
+        """
+        return array.array(self.array_type, self._data)
+
+    @property
+    def array_type(self):
+        return get_array_type(self.sample_width * 8)
+
     def __len__(self):
         """
         returns the length of this audio segment in milliseconds
@@ -198,6 +275,15 @@ class AudioSegment(object):
         else:
             return self.apply_gain(arg)
 
+    def __radd__(self, rarg):
+        """
+        Permit use of sum() builtin with an iterable of AudioSegments
+        """
+        if rarg == 0:
+            return self
+        raise TypeError("Gains must be the second addend after the "
+                        "AudioSegment")
+
     def __sub__(self, arg):
         if isinstance(arg, AudioSegment):
             raise TypeError("AudioSegment objects can't be subtracted from "
@@ -247,7 +333,7 @@ class AudioSegment(object):
             'channels': self.channels
         }
         metadata.update(overrides)
-        return AudioSegment(data=data, metadata=metadata)
+        return self.__class__(data=data, metadata=metadata)
 
     @classmethod
     def _sync(cls, seg1, seg2):
@@ -265,9 +351,6 @@ class AudioSegment(object):
         seg1 = seg1.set_sample_width(sample_width)
         seg2 = seg2.set_sample_width(sample_width)
 
-        assert(len(seg1) == s1_len)
-        assert(len(seg2) == s2_len)
-
         return seg1, seg2
 
     def _parse_position(self, val):
@@ -287,13 +370,11 @@ class AudioSegment(object):
         })
 
     @classmethod
-    def silent(cls, duration=1000):
+    def silent(cls, duration=1000, frame_rate=11025):
         """
         Generate a silent audio segment.
-        duration specified in milliseconds (default: 1000ms).
+        duration specified in milliseconds (default duration: 1000ms, default frame_rate: 11025).
         """
-        # lowest frame rate I've seen in actual use
-        frame_rate = 11025
         frames = int(frame_rate * (duration / 1000.0))
         data = b"\0\0" * frames
         return cls(data, metadata={"channels": 1,
@@ -302,18 +383,38 @@ class AudioSegment(object):
                                    "frame_width": 2})
 
     @classmethod
-    def from_file(cls, file, format=None):
+    def from_file(cls, file, format=None, **kwargs):
         orig_file = file
         file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
 
         if format:
+            format = format.lower()
             format = AUDIO_FILE_EXT_ALIASES.get(format, format)
 
-        if format == "wav" or (isinstance(orig_file, basestring) and orig_file.endswith(".wav")):
+        def is_format(f):
+            f = f.lower()
+            if format == f:
+                return True
+            if isinstance(orig_file, basestring):
+                return orig_file.lower().endswith(".{0}".format(f))
+            return False
+
+        if is_format("wav"):
             try:
                 return cls._from_safe_wav(file)
             except:
                 file.seek(0)
+        elif is_format("raw") or is_format("pcm"):
+            sample_width = kwargs['sample_width']
+            frame_rate = kwargs['frame_rate']
+            channels = kwargs['channels']
+            metadata = {
+                'sample_width': sample_width,
+                'frame_rate': frame_rate,
+                'channels': channels,
+                'frame_width': channels * sample_width
+            }
+            return cls(data=file.read(), metadata=metadata)
 
         input_file = NamedTemporaryFile(mode='wb', delete=False)
         input_file.write(file.read())
@@ -321,28 +422,29 @@ class AudioSegment(object):
 
         output = NamedTemporaryFile(mode="rb", delete=False)
 
-        convertion_command = [cls.converter,
+        conversion_command = [cls.converter,
                               '-y',  # always overwrite existing files
                               ]
 
         # If format is not defined
         # ffmpeg/avconv will detect it automatically
         if format:
-            convertion_command += ["-f", format]
+            conversion_command += ["-f", format]
 
-        convertion_command += [
+        conversion_command += [
             "-i", input_file.name,  # input_file options (filename last)
             "-vn",  # Drop any video streams if there are any
             "-f", "wav",  # output options (filename last)
             output.name
         ]
-        
-        log_conversion(convertion_command)
 
-        retcode = subprocess.call(convertion_command, stderr=open(os.devnull))
+        log_conversion(conversion_command)
 
-        if retcode != 0:
-            raise CouldntDecodeError("Decoding failed. ffmpeg returned error code: {0}".format(retcode))
+        p = subprocess.Popen(conversion_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p_out, p_err = p.communicate()
+
+        if p.returncode != 0:
+            raise CouldntDecodeError("Decoding failed. ffmpeg returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(p.returncode, p_err))
 
         obj = cls._from_safe_wav(output)
 
@@ -369,6 +471,10 @@ class AudioSegment(object):
     def from_wav(cls, file):
         return cls.from_file(file, 'wav')
 
+    @classmethod
+    def from_raw(cls, file, **kwargs):
+        return cls.from_file(file, 'raw', sample_width=kwargs['sample_width'], frame_rate=kwargs['frame_rate'], channels=kwargs['channels'])
+
     @classmethod
     def _from_safe_wav(cls, file):
         file = _fd_or_path_or_tempfile(file, 'rb', tempfile=False)
@@ -384,15 +490,15 @@ class AudioSegment(object):
 
         format (string)
             Format for destination audio file.
-            ('mp3', 'wav', 'ogg' or other ffmpeg/avconv supported files)
+            ('mp3', 'wav', 'raw', 'ogg' or other ffmpeg/avconv supported files)
 
         codec (string)
             Codec used to encoding for the destination.
 
         bitrate (string)
             Bitrate used when encoding destination file. (64, 92, 128, 256, 312k...)
-            Each codec accepts different bitrate arguments so take a look at the 
-            ffmpeg documentation for details (bitrate usually shown as -b, -ba or 
+            Each codec accepts different bitrate arguments so take a look at the
+            ffmpeg documentation for details (bitrate usually shown as -b, -ba or
             -a:b).
 
         parameters (string)
@@ -410,6 +516,11 @@ class AudioSegment(object):
         out_f = _fd_or_path_or_tempfile(out_f, 'wb+')
         out_f.seek(0)
 
+        if format == "raw":
+            out_f.write(self._data)
+            out_f.seek(0)
+            return out_f
+
         # for wav output we can just write the data directly to out_f
         if format == "wav":
             data = out_f
@@ -433,7 +544,7 @@ class AudioSegment(object):
         output = NamedTemporaryFile(mode="w+b", delete=False)
 
         # build converter command to export
-        convertion_command = [
+        conversion_command = [
             self.converter,
             '-y',  # always overwrite existing files
             "-f", "wav", "-i", data.name,  # input options (filename last)
@@ -444,14 +555,14 @@ class AudioSegment(object):
 
         if codec is not None:
             # force audio encoder
-            convertion_command.extend(["-acodec", codec])
+            conversion_command.extend(["-acodec", codec])
 
         if bitrate is not None:
-            convertion_command.extend(["-b:a", bitrate])
+            conversion_command.extend(["-b:a", bitrate])
 
         if parameters is not None:
             # extend arguments with arbitrary set
-            convertion_command.extend(parameters)
+            conversion_command.extend(parameters)
 
         if tags is not None:
             if not isinstance(tags, dict):
@@ -460,7 +571,7 @@ class AudioSegment(object):
                 # Extend converter command with tags
                 # print(tags)
                 for key, value in tags.items():
-                    convertion_command.extend(
+                    conversion_command.extend(
                         ['-metadata', '{0}={1}'.format(key, value)])
 
                 if format == 'mp3':
@@ -468,21 +579,25 @@ class AudioSegment(object):
                     if id3v2_version not in id3v2_allowed_versions:
                         raise InvalidID3TagVersion(
                             "id3v2_version not allowed, allowed versions: %s" % id3v2_allowed_versions)
-                    convertion_command.extend([
+                    conversion_command.extend([
                         "-id3v2_version",  id3v2_version
                     ])
 
-        convertion_command.extend([
+        if sys.platform == 'darwin':
+            conversion_command.extend(["-write_xing", "0"])
+
+        conversion_command.extend([
             "-f", format, output.name,  # output options (filename last)
         ])
-        
-        log_conversion(convertion_command)
-        
+
+        log_conversion(conversion_command)
+
         # read stdin / write stdout
-        subprocess.call(convertion_command,
-                        # make converter shut up
-                        stderr=open(os.devnull)
-                        )
+        p = subprocess.Popen(conversion_command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p_out, p_err = p.communicate()
+
+        if p.returncode != 0:
+            raise CouldntEncodeError("Encoding failed. ffmpeg/avlib returned error code: {0}\n\nOutput from ffmpeg/avlib:\n\n{1}".format(p.returncode, p_err))
 
         output.seek(0)
         out_f.write(output.read())
@@ -587,7 +702,7 @@ class AudioSegment(object):
     def dBFS(self):
         rms = self.rms
         if not rms:
-            return - float("infinity")
+            return -float("infinity")
         return ratio_to_db(self.rms / self.max_possible_amplitude)
 
     @property
@@ -601,7 +716,7 @@ class AudioSegment(object):
 
         # since half is above 0 and half is below the max amplitude is divided
         return max_possible_val / 2
-        
+
     @property
     def max_dBFS(self):
         return ratio_to_db(self.max, self.max_possible_amplitude)
@@ -807,5 +922,15 @@ class AudioSegment(object):
             data=audioop.reverse(self._data, self.sample_width)
         )
 
+    def _repr_html_(self):
+            src = """
+                    <audio controls>
+                        <source src="data:audio/mpeg;base64,{base64}" type="audio/mpeg"/>
+                        Your browser does not support the audio element.
+                    </audio>
+                  """
+            fh = self.export()
+            data = base64.b64encode(fh.read()).decode('ascii')
+            return src.format(base64=data)
 
 from . import effects
