@@ -83,6 +83,10 @@ def ratio_to_db(ratio, val2=None, using_amplitude=True):
     # accept 2 values and use the ratio of val1 to val2
     if val2 is not None:
         ratio = ratio / val2
+    
+    # special case for multiply-by-zero (convert to silence)
+    if ratio == 0:
+        return -float('inf')
 
     if using_amplitude:
         return 20 * log(ratio, 10)
@@ -140,7 +144,7 @@ def which(program):
     if os.name == "nt" and not program.endswith(".exe"):
         program += ".exe"
 
-    envdir_list = os.environ["PATH"].split(os.pathsep)
+    envdir_list = [os.curdir] + os.environ["PATH"].split(os.pathsep)
 
     for envdir in envdir_list:
         program_path = os.path.join(envdir, program)
@@ -195,11 +199,14 @@ def mediainfo(filepath):
 
     from .audio_segment import AudioSegment
 
-    command = "{0} -v quiet -show_format -show_streams {1}".format(
+    command = [
         get_prober_name(),
+        "-v", "quiet",
+        "-show_format",
+        "-show_streams",
         filepath
-    )
-    output = Popen(command.split(), stdout=PIPE).communicate()[0].decode("utf-8")
+    ]
+    output = Popen(command, stdout=PIPE).communicate()[0].decode("utf-8")
 
     rgx = re.compile(r"(?:(?P<inner_dict>.*?):)?(?P<key>.*?)\=(?P<value>.*?)$")
     info = {}
