@@ -3,6 +3,7 @@ import mimetypes
 import os
 import unittest
 from tempfile import NamedTemporaryFile
+import struct
 
 from pydub import AudioSegment
 from pydub.utils import (
@@ -17,6 +18,7 @@ from pydub.exceptions import (
     InvalidID3TagVersion,
     InvalidDuration,
     CouldntDecodeError,
+    MissingAudioParameter,
 )
 from pydub.silence import (
     detect_silence,
@@ -64,7 +66,7 @@ class FileAccessTests(unittest.TestCase):
         self.assertTrue(len(seg1) > 0)
 
 
-test1wav = test1 = test2 = test3 = testparty = None
+test1wav = test4wav = test1 = test2 = test3 = testparty = None
 
 
 class AudioSegmentTests(unittest.TestCase):
@@ -100,6 +102,26 @@ class AudioSegmentTests(unittest.TestCase):
         upper_bound = val + tolerance
         self.assertWithinRange(val, lower_bound, upper_bound)
 
+    def test_direct_instantiation_with_bytes(self):
+        seg = AudioSegment(b'RIFF\x28\x00\x00\x00WAVEfmt \x10\x00\x00\x00\x01\x00\x02\x00\x00}\x00\x00\x00\xf4\x01\x00\x04\x00\x10\x00data\x04\x00\x00\x00\x00\x00\x00\x00')
+        self.assertEqual(seg.frame_count(), 1)
+        self.assertEqual(seg.channels, 2)
+        self.assertEqual(seg.sample_width, 2)
+        self.assertEqual(seg.frame_rate, 32000)
+
+    def test_24_bit_audio(self):
+        path24 = os.path.join(data_dir, 'test1-24bit.wav')
+        seg24 = AudioSegment._from_safe_wav(path24)
+        # The data length lies at bytes 40-44
+        with open(path24, 'rb') as f:
+            raw24 = f.read()
+        len24 = struct.unpack("<L", raw24[40:44])[0]
+
+        # should have been converted to 32 bit
+        self.assertEqual(seg24.sample_width, 4)
+        # the data length should have grown by exactly 4:3 (24 bits turn into 32 bits)
+        self.assertEqual(len(seg24.raw_data) * 3, len24 * 4)
+
     def test_concat(self):
         catted_audio = self.seg1 + self.seg2
 
@@ -113,6 +135,20 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertEqual(len(merged1), len(self.seg1) + len(self.seg3) - 100)
         self.assertEqual(len(merged2), len(self.seg2) + len(self.seg3) - 100)
 
+    def test_sum(self):
+        def gen():
+            yield self.seg1
+            yield self.seg2
+            yield self.seg3
+
+        try:
+            summed = sum(gen())
+        except TypeError as e:
+            if "unsupported operand" in str(e):
+                self.fail("Could not sum() audio segments.")
+            else:
+                raise
+
     def test_volume_with_add_sub(self):
         quieter = self.seg1 - 6
         self.assertAlmostEqual(ratio_to_db(quieter.rms, self.seg1.rms),
@@ -228,7 +264,7 @@ class AudioSegmentTests(unittest.TestCase):
         for part in short:
             rebuilt1 += part
 
-        rebuilt2 = sum([part for part in short], short[:0])
+        rebuilt2 = sum([part for part in short])
 
         self.assertTrue(short._data == rebuilt1._data)
         self.assertTrue(short._data == rebuilt2._data)
@@ -262,13 +298,74 @@ class AudioSegmentTests(unittest.TestCase):
 
         self.assertEqual(len(seg_lchannel), len(seg))
         self.assertEqual(len(seg_rchannel), len(seg))
-        
+
         self.assertEqual(seg_lchannel.frame_rate, seg.frame_rate)
         self.assertEqual(seg_rchannel.frame_rate, seg.frame_rate)
 
         self.assertEqual(seg_lchannel.frame_count(), seg.frame_count())
         self.assertEqual(seg_rchannel.frame_count(), seg.frame_count())
 
+
+    def test_apply_gain_stereo(self):
+        seg = self.seg1
+
+        orig_l, orig_r = seg.split_to_mono()
+        orig_dbfs_l = orig_l.dBFS
+        orig_dbfs_r = orig_r.dBFS
+
+        # for readability: infinity
+        inf = float("inf")
+
+        def assertAlmostEqual(v1, v2, **kwargs):
+            if v1 in (inf, -inf):
+                self.assertEqual(v1, v2)
+            else:
+                self.assertAlmostEqual(v1, v2, **kwargs)
+
+        def check_stereo_gain(left_dbfs_change, right_dbfs_change):
+            panned = seg.apply_gain_stereo(left_dbfs_change, right_dbfs_change)
+            self.assertEqual(panned.channels, 2)
+
+            l, r = panned.split_to_mono()
+            assertAlmostEqual(l.dBFS, orig_dbfs_l + left_dbfs_change, places=2)
+            assertAlmostEqual(r.dBFS, orig_dbfs_r + right_dbfs_change, places=2)
+
+        # hard left
+        check_stereo_gain(0.0, -inf)
+        check_stereo_gain(0.0, -6.0)
+        check_stereo_gain(0.0, 0.0)
+        check_stereo_gain(-6.0, 0.0)
+        check_stereo_gain(-inf, 0.0)
+
+    def test_pan(self):
+        seg = self.seg1
+
+        orig_l, orig_r = seg.split_to_mono()
+        orig_dbfs_l = orig_l.dBFS
+        orig_dbfs_r = orig_r.dBFS
+
+        # for readability: infinity
+        inf = float("inf")
+
+        def assertAlmostEqual(v1, v2, **kwargs):
+            if v1 in (inf, -inf):
+                self.assertEqual(v1, v2)
+            else:
+                self.assertAlmostEqual(v1, v2, **kwargs)
+
+        def check_pan(pan, left_dbfs_change, right_dbfs_change):
+            panned = seg.pan(pan)
+
+            l, r = panned.split_to_mono()
+            assertAlmostEqual(l.dBFS, orig_dbfs_l + left_dbfs_change, places=1)
+            assertAlmostEqual(r.dBFS, orig_dbfs_r + right_dbfs_change, places=1)
+
+        check_pan(-1.0, 3.0, -inf)
+        check_pan(-0.5, 1.5, -4.65)
+        check_pan(0.0, 0.0, 0.0)
+        check_pan(0.5, -4.65, 1.5)
+        check_pan(1.0, -inf, 3.0)
+
     def test_export_as_mp3(self):
         seg = self.seg1
         exported_mp3 = seg.export()
@@ -287,6 +384,15 @@ class AudioSegmentTests(unittest.TestCase):
                                    len(seg),
                                    percentage=0.01)
 
+    def test_export_as_raw(self):
+        seg = self.seg1
+        exported_raw = seg.export(format='raw')
+        seg_exported_raw = AudioSegment.from_raw(exported_raw, sample_width=seg.sample_width, frame_rate=seg.frame_rate, channels = seg.channels)
+
+        self.assertWithinTolerance(len(seg_exported_raw),
+                                   len(seg),
+                                   percentage=0.01)
+
     def test_export_as_ogg(self):
         seg = self.seg1
         exported_ogg = seg.export(format='ogg')
@@ -579,15 +685,36 @@ class AudioSegmentTests(unittest.TestCase):
         self.assertAlmostEqual(-0.0, sine_0_dbfs.max_dBFS, 2)
         self.assertAlmostEqual(-3.0, sine_minus_3_dbfs.max_dBFS, 2)
 
+    def test_array_type(self):
+        self.assertEqual(self.seg1.array_type, "h")
+        self.assertEqual(self.seg2.array_type, "h")
+        self.assertEqual(self.seg3.array_type, "h")
+        self.assertEqual(self.mp3_seg_party.array_type, "h")
+
+        silence = AudioSegment.silent(50)
+        self.assertEqual(silence.array_type, "h")
+        self.assertEqual(silence.set_sample_width(1).array_type, "b")
+        self.assertEqual(silence.set_sample_width(4).array_type, "i")
+
+    def test_sample_array(self):
+        samples = Sine(450).to_audio_segment().get_array_of_samples()
+        self.assertEqual(
+            list(samples[:8]),
+            [0, 2099, 4190, 6263, 8311, 10325, 12296, 14217]
+        )
+
 
 class SilenceTests(unittest.TestCase):
 
     def setUp(self):
-        global test1wav
+        global test1wav, test4wav
         if not test1wav:
             test1wav = AudioSegment.from_wav(os.path.join(data_dir, 'test1.wav'))
+        if not test4wav:
+            test4wav = AudioSegment.from_wav(os.path.join(data_dir, 'test4.wav'))
 
         self.seg1 = test1wav
+        self.seg4 = test4wav
 
     def test_detect_completely_silent_segment(self):
         seg = AudioSegment.silent(5000)
@@ -608,6 +735,14 @@ class SilenceTests(unittest.TestCase):
         silent_ranges = detect_silence(self.seg1, min_silence_len=500, silence_thresh=-20)
         self.assertEqual(silent_ranges, [[0, 775], [3141, 4033], [5516, 6051]])
 
+    def test_realistic_audio(self):
+        silent_ranges = detect_silence(self.seg4, min_silence_len=1000, silence_thresh=self.seg4.dBFS)
+
+        prev_end = -1
+        for start, end in silent_ranges:
+            self.assertTrue(start > prev_end)
+            prev_end = end
+
 
 class GeneratorTests(unittest.TestCase):
 
@@ -644,7 +779,9 @@ class NoConverterTests(unittest.TestCase):
 
     def setUp(self):
         self.wave_file = os.path.join(data_dir, 'test1.wav')
+        self.wave24_file = os.path.join(data_dir, 'test1-24bit.wav')
         self.mp3_file = os.path.join(data_dir, 'test1.mp3')
+        self.raw_file = os.path.join(data_dir, 'test1.raw')
         AudioSegment.converter = "definitely-not-a-path-to-anything-asdjklqwop"
 
     def tearDown(self):
@@ -663,6 +800,33 @@ class NoConverterTests(unittest.TestCase):
         seg = AudioSegment.from_file(self.wave_file, format="wav")
         self.assertTrue(len(seg) > 1000)
 
+    def test_opening_wav24_file(self):
+        seg = AudioSegment.from_wav(self.wave24_file)
+        self.assertTrue(len(seg) > 1000)
+
+        seg = AudioSegment.from_file(self.wave24_file)
+        self.assertTrue(len(seg) > 1000)
+
+        seg = AudioSegment.from_file(self.wave24_file, "wav")
+        self.assertTrue(len(seg) > 1000)
+
+        seg = AudioSegment.from_file(self.wave24_file, format="wav")
+        self.assertTrue(len(seg) > 1000)
+
+    def test_opening_raw_file(self):
+        seg = AudioSegment.from_raw(self.raw_file, sample_width=2, frame_rate=32000, channels=2)
+        self.assertTrue(len(seg) > 1000)
+
+        seg = AudioSegment.from_file(self.raw_file, "raw", sample_width=2, frame_rate=32000, channels=2)
+        self.assertTrue(len(seg) > 1000)
+
+        seg = AudioSegment.from_file(self.raw_file, format="raw", sample_width=2, frame_rate=32000, channels=2)
+        self.assertTrue(len(seg) > 1000)
+
+    def test_opening_raw_file_with_missing_args_fails(self):
+        func = partial(AudioSegment.from_raw, self.raw_file)
+        self.assertRaises(KeyError, func)
+
     def test_opening_mp3_file_fails(self):
         func = partial(AudioSegment.from_mp3, self.mp3_file)
         self.assertRaises(OSError, func)
@@ -676,6 +840,34 @@ class NoConverterTests(unittest.TestCase):
         func = partial(AudioSegment.from_file, self.mp3_file, format="mp3")
         self.assertRaises(OSError, func)
 
+    def test_init_AudioSegment_data_buffer(self):
+        seg = AudioSegment(data = "\0" * 34, sample_width=2, frame_rate=4, channels=1)
+
+        self.assertEqual(seg.duration_seconds, 4.25)
+
+        self.assertEqual(seg.sample_width, 2)
+
+        self.assertEqual(seg.frame_rate, 4)
+
+
+    def test_init_AudioSegment_data_buffer_with_missing_args_fails(self):
+
+        func = partial(AudioSegment, data = "\0" * 16, sample_width=2, frame_rate=2)
+        self.assertRaises(MissingAudioParameter, func)
+
+        func = partial(AudioSegment, data = "\0" * 16, sample_width=2, channels=1)
+        self.assertRaises(MissingAudioParameter, func)
+
+        func = partial(AudioSegment, data = "\0" * 16, frame_rate=2, channels=1)
+        self.assertRaises(MissingAudioParameter, func)
+
+
+    def test_init_AudioSegment_data_buffer_with_bad_values_fails(self):
+
+        func = partial(AudioSegment, data = "\0" * 14, sample_width=4, frame_rate=2, channels=1)
+        self.assertRaises(ValueError, func)
+
+
     def test_exporting(self):
         seg = AudioSegment.from_wav(self.wave_file)
         exported = AudioSegment.from_wav(seg.export(format="wav"))
